<?php
namespace Ecommerce\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\GER\Utilities\Environment;

class LoginService
{

    const SITUACAO_ATIVO = 'A';

    protected $entityManager;
    protected $criptografia;
    protected $smtpUtil;

    public function __construct(EntityManager $entityManager, Criptografia $criptografia, Environment $environment, $smtpUtil)
    {
        $this->entityManager = $entityManager;
        $this->criptografia = $criptografia;
        $this->environment = $environment;
        $this->smtpUtil = $smtpUtil;
    }

    private function retornaEmpresa($cdforneced)
    {

        return $this->entityManager->getRepository('\Geral\Entities\Fornecedor')
            ->findOneBy(array('cdforneced' => $cdforneced));

    }

    private function iniciaAmbiente($dadosSessao)
    {

        $this->environment->setCdforneced($dadosSessao['CDFORNECED']);
        $this->environment->setDslogineco($dadosSessao['DSLOGINECO']);
        $this->environment->setIdacessocot($dadosSessao['IDACESSOCOT']);
        $this->environment->setIdacessoaf($dadosSessao['IDACESSOAF']);
        $this->environment->setIdacessocr($dadosSessao['IDACESSOCR']);
        $this->environment->setIdadministrador($dadosSessao['IDADMINISTRADOR']);

    }

    private function criaDadosDaSessao($fornecedor, $empresa)
    {

        return array(
            "CDFORNECED" => $fornecedor->getCdforneced(),
            "DSLOGINECO" => $fornecedor->getDslogineco(),
            "IDACESSOCOT" => $fornecedor->getIdacessocot(),
            "IDACESSOAF" => $fornecedor->getIdacessoaf(),
            "IDACESSOCR" => $fornecedor->getIdacessocr(),
            "IDADMINISTRADOR" => $fornecedor->getIdadministrador(),
            "NMFANTFORN" => $empresa->getNmfantforn(),
            "NMRAZSOCFORN" => $empresa->getNmrazsocforn(),
            "NRINSJURFORN" => $empresa->getNrinsjurforn(),
            "IDTPIJURFORN" => $empresa->getIdtpijurforn()
        );

    }

    public function validaSenha($cdsenhaweb, $senha)
    {

        if ($cdsenhaweb !== $this->criptografia->retornaSenhaWeb($senha)) {
            throw new \Exception("Senha inválida.");
        }

    }

    private function retornaFornecedorPorEmail($email)
    {

        $fornecedor = $this->entityManager->getRepository('\Portalecommerce\Entities\Dadosfornecommerce')
            ->findBy(array('dslogineco' => $email));

        if (!$fornecedor) {
            throw new \Exception("Login não está cadastrado.");
        }

        return $fornecedor;

    }

    private function retornaFornecedorEcommerce($cdfornecedor, $email)
    {

        $fornecedor = $this->entityManager->getRepository('\Portalecommerce\Entities\Dadosfornecommerce')
            ->findOneBy(array('cdforneced' => $cdfornecedor, 'dslogineco' => $email));

        if (!$fornecedor) {
            throw new \Exception("Login não está cadastrado.");
        }

        return $fornecedor;

    }

    private function retornaFornecedorFilho($cdfornecedor)
    {

        $filhos = $this->entityManager->getRepository('\Geral\Entities\Fornecedor')
            ->findBy(array('cdfornerelacaf' => $cdfornecedor));

        $empresa = $this->entityManager->getRepository('\Geral\Entities\Fornecedor')
            ->findOneBy(array('cdforneced' => $cdfornecedor));

        $nrinsjurforn = $empresa->getNrinsjurforn();
        $nmrazsocforn = $empresa->getNmrazsocforn();
        $idtpijurforn = $empresa->getIdtpijurforn();

        $filhosArray = array();

        $filhosArray[] = array(
            'CDFORNECEDOR' => $this->fornecedor->getCdforneced(),
            'NRINSJURFORN' => $nrinsjurforn,
            'NMRAZSOCFORN' => $nmrazsocforn,
            'IDTPIJURFORN' => $idtpijurforn,
            'INSJURRAZSOC' => $nrinsjurforn . ' - ' . $nmrazsocforn
        );

        foreach ($filhos as $filho) {

            $nrinsjurfornFilho = $filho->getNrinsjurforn();
            $nmrazsocfornFilho = $filho->getNmrazsocforn();
            $idtpijurfornFilho = $empresa->getIdtpijurforn();

            $filhosArray[] = array(
                'CDFORNECEDOR' => $filho->getCdforneced(),
                'NRINSJURFORN' => $nrinsjurfornFilho,
                'NMRAZSOCFORN' => $nmrazsocfornFilho,
                'IDTPIJURFORN' => $idtpijurfornFilho,
                'INSJURRAZSOC' => $nrinsjurfornFilho . ' - ' . $nmrazsocfornFilho
            );

        }

        return $filhosArray;

    }

    public function autenticar($cdfornecedor, $email, $senha)
    {

        $this->fornecedor = $this->retornaFornecedorEcommerce($cdfornecedor, $email);

        $this->validaSenha($this->fornecedor->getCdsenhaweb(), $senha);

        $empresa = $this->retornaEmpresa($this->fornecedor->getCdforneced());

        $dadosDaSessao = $this->criaDadosDaSessao($this->fornecedor, $empresa);

        $fornecedorFilho = $this->retornaFornecedorFilho($this->fornecedor->getCdforneced());

        $parametros = $this->retornaParamGeral($empresa->getNrorg());

        $this->iniciaAmbiente($dadosDaSessao);

        return array($dadosDaSessao, $fornecedorFilho, $parametros);

    }

    private function criaEmail($fornecedor)
    {

        $cdrandomacesso = $fornecedor->getCdrandomacesso();
        $email = $fornecedor->getDslogineco();

        return array(
            "assunto" => "Portal E-Commerce - Redefinição de senha.",
            "email" => $email,
            "nome" => $email,
            "mensagem" => "Olá\n\nRecebemos uma solicitação de redefinição de senha para sua conta no PortalECommerce. Para redefinir sua senha, informe o código abaixo:\n\n$cdrandomacesso\n\nSe você não solicitou uma redefinição de senha, desconsidere este email.\n\nA equipe do Portal E-Commerce."
        );

    }

    private function atualizarCodigoVerificacao($fornecedores, $codigoVerificacao)
    {
        foreach ($fornecedores as $fornecedor) {
            $fornecedor->setCdrandomacesso($codigoVerificacao);
            $this->entityManager->persist($fornecedor);
        }
        $this->entityManager->flush();
    }

    private function retornaCodigoVerificacao($length = 10)
    {

        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;

    }

    public function checaEmail($email)
    {

        $fornecedor = $this->retornaFornecedorPorEmail($email);

        $codigoVerificacao = $this->retornaCodigoVerificacao(6);

        $this->atualizarCodigoVerificacao($fornecedor, $codigoVerificacao);

        $email = $this->criaEmail($fornecedor[0]);
        
        $this->smtpUtil->setFromName('Portal E-Commerce');
        $this->smtpUtil->setSubject($email['assunto']);
        $this->smtpUtil->setNmAddress($email['nome']);
        $this->smtpUtil->sendMail($email['mensagem'], $email['email']);
        
        return $fornecedor;

    }

    private function verificaCodigo($codigoSalvo, $codigoVerificacao)
    {

        if ($codigoSalvo !== $codigoVerificacao) {
            throw new \Exception("Código inválido.");
        }

    }

    public function addNovaSenha($email, $codigoVerificacao, $novaSenha)
    {

        $fornecedores = $this->retornaFornecedorPorEmail($email);

        $this->verificaCodigo($fornecedores[0]->getCdrandomacesso(), $codigoVerificacao);

        $this->salvaSenha($fornecedores, $novaSenha);

        return true;

    }

    public function alteraSenha($email, $novaSenha)
    {

        $fornecedores = $this->retornaFornecedorPorEmail($email);

        $this->salvaSenha($fornecedores, $novaSenha);

    }

    private function salvaSenha($fornecedores, $novaSenha)
    {
        foreach ($fornecedores as $fornecedor) {
            $fornecedor->setCdsenhaweb($this->criptografia->retornaSenhaWeb($novaSenha));
            $fornecedor->setCdsenha($this->criptografia->retornaSenhaDelphi($novaSenha));
            $this->entityManager->persist($fornecedor);
        }
        $this->entityManager->flush();
    }

    public function buscaUsuarioLogado($email)
    {

        $fornecedor = $this->retornaFornecedorPorEmail($email);

        return $fornecedor;

    }

    private function retornaParamGeral($nrorg)
    {


        $param = $this->entityManager->getRepository('Geral\Entities\Paramgeral')->findOneBy(array('nrorg' => $nrorg));


        $idsisdos = $param->getIdsisdos();
        $nrorg = $param->getNrorg();
        $cdpictprod = $param->getCdpictprod();

        $parametrosArray = array(
            'IDSISDOS' => $idsisdos,
            'NRORG' => $nrorg,
            'CDPICTPROD' => $cdpictprod
        );

        return $parametrosArray;

    }

}