let Dashboard =  function(General, DashboardService, engine, localStorageEngine, ScreenService, templateManager, ZHPromise) {
	/* global $ */ /* global getLocalVar */ /* global Configuration */
	var self = this;

	this.load = function() {
        let templateUrl = {
            package: "warningMessages",
            baseUrl: "./assets/templates/html/warningMessages.html"
        };
        
        window.templateUrls.push(templateUrl);
    };
    
    this.removeIdModal = function() {
    	$('section.popup-container').removeAttr('id');
		$('section.popup-content div.container[zh-scroller]').removeAttr('id');
		$('section.popup-content div.container[data-zh-scroller]').removeAttr('id');
    };
    
    this.setIdModal = function () {
    	$('section.popup-container').attr('id', 'popUpContainer');
		$('section.popup-content div.container[zh-scroller]').attr('id', 'popUpContent');
		$('section.popup-content div.container[data-zh-scroller]').attr('id', 'popUpContent');
    };
    	
    this.closePopUp = function() {
    	self.removeIdModal();
    	ScreenService.closePopup(true);
    };
    
    this.accessControl = function(access) {
		if (getLocalVar(access) == 'S') {
			return true;
		}
		
		return false;
	};
    
	this.iniciaDashboard = function() {
		let interval = setInterval(function() {
			if (getLocalVar('INICIA_DASHBOARD') === true) {
				if (templateManager.project.groupMenu[0].menus.length > 0) {
					clearInterval(interval);
					
					General.callRestEngine("product#/verificaIdsisdos", "DataSet", []).then(function(callback) {
						const idSisDos = callback.dataset.resp.IDSISDOS;
						self.loadComponents(idSisDos);
						self.habilitaTrocaFornecedor();
					});
				} else if(getLocalVar('INICIA_DASHBOARD') === false) {
					clearInterval(interval);
				}
			}
		}, 300);
	};
	
	this.getComponentsAccess = function(idSisDos) {
		const accessPC = self.accessControl('IDACESSOCOT');
		const accessAF = self.accessControl('IDACESSOAF');
		const accessCR = self.accessControl('IDACESSOCR');
		const accessMP = self.accessControl('IDACESSOUPLOAD');
		
		let menuNameCR = "contas_a_receber";
		let windowNameCR = "product#ContasAReceberTek";
		let methodCR = 'dadosCR';
		let removeMenuNameCR = "contas_a_receber_GR";
		
		if(idSisDos == 3) {
			menuNameCR = "contas_a_receber_GR";
			windowNameCR = "product#ContasAReceber";
			methodCR = 'dadosContaGR';
			removeMenuNameCR = "contas_a_receber";
		}
		
		return [
			{
				"widgetName": "pedidoDeCotacao",
				"menuName": "pedidos_de_cotacao",
				"windowName": "product#Pedido_Cotacao",
				"isVisible": accessPC,
				"method": 'dadosPedCol'
			},
			{
				"widgetName": "autoFornecimento",
				"menuName": "autorizacao_de_fornecimento",
				"windowName": "product#AutorizacaoFornecimento",
				"isVisible": accessAF,
				"method": 'dadosAFNLidas'
			},
			{
				"widgetName": "contasReceber",
				"menuName": menuNameCR,
				"windowName": windowNameCR,
				"isVisible": accessCR,
				"method": methodCR,
				"removeMenuName": removeMenuNameCR
			},
			{
				"widgetName": "meusProdutos",
				"menuName": "meus_produtos",
				"windowName": "product#MeusProdutos",
				"isVisible": accessMP,
				"method": 'dadosMeusProdutos'
			}
		];
	};
	
	this.setVisibilityMenu = function(menuName, visibility) {
		const menu = templateManager.project.getMenu(menuName);
		
		if (typeof(menu) != "undefined") {
			menu.isVisible = visibility;
		} else {
			console.error("Verificar se a tela '" + menuName + "' está disponível para a organização.");
		}
	};
	
	this.setVisibilityAdmin = function() {
		const adminAccess = self.accessControl('IDADMINISTRADOR');
		if(!adminAccess) {
			templateManager.project.removeMenu('administrador');
		}
	};
	
	this.removeDefaultMenu = function() {
		templateManager.project.removeMenu('parametrizacao');
		templateManager.project.removeMenu('acl');
		templateManager.project.removeMenu('log');
		templateManager.project.removeMenu('zhHistory');
	};
	
	this.setAFData = function(valuesAF) {
		let message = '';
				
		if (valuesAF.AFNLIDAS <= 1) {
			message = ScreenService.i18n(" AF disponível");
		} else {
			message = ScreenService.i18n(" AFs disponíveis");
		}
	
		templateManager.container.getWidget('autoFornecimento').notification = valuesAF.AFNLIDAS + message;
	};
	
	this.setCRData = function(valuesCR) {
		const totalCR = parseFloat(valuesCR.VRVENC) + parseFloat(valuesCR.VRATUAL) + parseFloat(valuesCR.VRAVENC);
		
		const dataCR = [
			{ "STATUS": ScreenService.i18n("Vencido"), "VALOR": !valuesCR.VRVENC || valuesCR.VRVENC === null ? '0' : valuesCR.VRVENC }, 
			{ "STATUS": ScreenService.i18n("Hoje"), "VALOR": !valuesCR.VRATUAL || valuesCR.VRATUAL === null ? '0' : valuesCR.VRATUAL }, 
			{ "STATUS": ScreenService.i18n("A Vencer"), "VALOR": !valuesCR.VRAVENC || valuesCR.VRATUAL === null ? '0' : valuesCR.VRAVENC }, 
			{ "STATUS": ScreenService.i18n("Total"), "VALOR": !valuesCR ? '0' : totalCR }
		];
		
		templateManager.container.getWidget('contasReceber').dataSource.data = dataCR;
	};
	
	this.setPCData = function(valuesPC) {
		const totalPC = parseInt(valuesPC.EXPHOJE) + parseInt(valuesPC.AEXPIRA);
		
		const dataPC = [
			{ "STATUS": ScreenService.i18n("Hoje"), "VALOR": !valuesPC.EXPHOJE ? '0' : valuesPC.EXPHOJE },
			{ "STATUS": ScreenService.i18n("A expirar"), "VALOR": !valuesPC.AEXPIRA ? '0' : valuesPC.AEXPIRA },
			{ "STATUS": ScreenService.i18n("Total"), "VALOR": !totalPC ? '0' : totalPC }
		];
		
		templateManager.container.getWidget('pedidoDeCotacao').dataSource.data = dataPC;
	};
	
	this.setMPData = function(valuesMP) {
		const dataMP = [
			{ "STATUS": ScreenService.i18n("C/ foto"), "VALOR": !valuesMP.QTDPRODFOTO ? '0' : valuesMP.QTDPRODFOTO },
			{ "STATUS": ScreenService.i18n("C/ descrição"), "VALOR": !valuesMP.QTDPRODDESC ? '0' : valuesMP.QTDPRODDESC },
			{ "STATUS": ScreenService.i18n("Total"), "VALOR": !valuesMP.TOTAL ? '0' : valuesMP.TOTAL }
		];
		
		templateManager.container.getWidget('meusProdutos').dataSource.data = dataMP;
	};
	
	this.setWMData = function(valuesWM) {
		const warningWidget = templateManager.container.getWidget('warningMessages');
			
		if (Object.keys(valuesWM).length === 0) {
			warningWidget.isVisible = false;
			warningWidget.currentRow.isEmpty = true;
		} else {
			valuesWM = self.removeHtmlTags(valuesWM);
			warningWidget.isVisible = true;
			warningWidget.currentRow = valuesWM;
		}
	};
	
	this.removeHtmlTags = function(messages) {
		const tmp = document.createElement("DIV");
		
		for (let message of messages) {
			tmp.innerHTML = message.TXCONTEUDOINF;
			message.TXCONTEUDOINFNOTAGS = tmp.textContent || tmp.innerText || "";
		}
		
		return messages;
	};
	
	this.promisesPush = function(methodName) {
		switch (methodName) {
			case 'dadosPedCol':
				return DashboardService.dadosPedCol();
			
			case 'dadosAFNLidas':
				return DashboardService.dadosAFNLidas();
			
			case 'dadosCR':
				return DashboardService.dadosCR();
			
			case 'dadosContaGR':
				return DashboardService.dadosContaGR();
				
			case 'dadosMeusProdutos':
				return DashboardService.dadosMeusProdutos();
		}	
	};
	
	this.loadComponents = function(idSisDos) {
		if (templateManager.project.groupMenu.length) {
			self.setVisibilityAdmin();
			self.removeDefaultMenu();
			
			const components = self.getComponentsAccess(idSisDos);
			let promises = [];
			
			for (let component of components) {
				if (component.isVisible) {
					promises.push(self.promisesPush(component.method));
					self.setVisibilityMenu(component.menuName, true);
				} else {
					self.setVisibilityMenu(component.menuName, false);
				}
				
				if (component.widgetName == 'contasReceber') {
					templateManager.project.removeMenu(component.removeMenuName);
				}
				
				let widget = templateManager.container.getWidget(component.widgetName);
				widget.windowName = component.windowName;
				widget.isVisible = component.isVisible;
				widget.refreshWidget = function(){};
			}
			
			promises.push(DashboardService.dadosWarningMessages());
			
			ZHPromise.all(promises).then(function(callbacks) {
				for (let callback of callbacks) {
					let callbackName = Object.keys(callback.dataset);
					let values = callback.dataset[callbackName];
					
					switch (callbackName[0]) {
						case 'autoForneNLidas':
							self.setAFData(values[0]);
						break;
						
						case 'dadosCR':
						case 'dadosContaGR':
							self.setCRData(values[0]);
						break;
						
						case 'dadosPedCol':
							self.setPCData(values[0]);
						break;
						
						case 'dadosMeusProdutos':
							self.setMPData(values[0]);
						break;
						
						case 'warningMessages':
							self.setWMData(values);
						break;
					}
				}
			});
		}
	};
	
	let containerToChangeSupplier = this.containerToChangeSupplier =  {containerName: 'product#FornecedoresLogin'} ;

	this.habilitaTrocaFornecedor = function() {
		const email = getLocalVar('EMAIL');
		
		engine.getSupplier(email).then(function(fornecedores) {
			fornecedores = fornecedores.dataset.fornecedoresLogin;
			
			if (fornecedores.length > 2) {
				$('.zh-user-action.ng-scope').css('display', 'block');
				containerToChangeSupplier.containerName = 'product#FornecedoresLogin';
			} else {
				DashboardService.fornecedoresRelacionados().then(function(callback) {
					if (callback.dataset.fornecedoresRelacionados.length > 0) {
						$('.zh-user-action.ng-scope').css('display', 'block');
						containerToChangeSupplier.containerName = 'product#TrocaFornecedor' ;
					} else {
						callback.dataset.fornecedoresRelacionados.length = 0;
						$('.zh-user-action.ng-scope').css('display', 'none');
					}
				});
			}
		});
	};
};

Configuration(function(ContextRegister) {
	ContextRegister.register('Dashboard', Dashboard);
});
